package TreeJuxtaposer;

import java.util.Iterator;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JButton;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JColorChooser;

import java.awt.GraphicsConfiguration;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Color;
import java.awt.Canvas;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;



/**
 * @author jslack
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class StateFrame extends JFrame implements ActionListener, MouseListener {

	private final static String title = "Groups";
	private JPanel blank;  // for spacing, if needed
	private TreeJuxtaposer tj;
	private static final int CANVAS_SIZE = 15;
	
	/* Active Panel
	 * Includes the following panels:
	 * 	grow direction
	 * 	grow action
	 * 	active group
	 */
	private JPanel activePanel;
	private final static JLabel activePanelLabel = new JLabel("Active");
	private JButton resetButton;
	
	/* grow direction panel */
	private JPanel growDirectionPanel;
	private ButtonGroup growSelect;
	private JRadioButton growMode[];
	private final static JLabel growDirLabel = new JLabel("Direction");
	private final static String growLabel[] = {"Horizontal", "Vertical", "Both"};
	private final static int H_MODE = 0;
	private final static int V_MODE = 1;
	private final static int B_MODE = 2;
	
	/* grow action panel */
	private JPanel growButtonPanel;
	private JButton growButton[];
	private final static JLabel growButtonLabel = new JLabel("Grow");
	private final static String buttonLabel[] = {"Bigger", "Smaller"};
	private final static int B_GROW = 0;
	private final static int S_GROW = 1;
	
	/* active group panel */
	private JPanel activeGroupPanel;
	private Canvas activeCanvas[];
	private ButtonGroup activeSelect;
	private JRadioButton activeMode[];
	public final static String activeLabel[] = {"Group A", "Group B", "Group C", "Group D",
																		"Mouse Over", "Differences", "Found", "LCA"};
	static final private String colorTitle[] = {
			"'Group A' color selection",
			"'Group B' color selection",
			"'Group C' color selection",
			"'Group D' color selection",
			"'Mouse Over' color selection",
			"'Tree Differences' color selection",
			"'Found Nodes' color selection",
			" 'LCA' color selection"
	};
	public final static int GA_ACT = 0;
	public final static int GB_ACT = 1;
	public final static int GC_ACT = 2;
	public final static int GD_ACT = 3;
	public final static int M_ACT = 4;
	public final static int D_ACT = 5;
	public final static int F_ACT = 6;
	public final static int LCA_ACT = 7;
	
	/* Mark panel */
	private JPanel markPanel;
	private final static JLabel markPanelLabel = new JLabel("Mark");
	
	/* Mark radio buttons */
	private ButtonGroup markSelect;
	private JRadioButton markMode[];
	private JPanel markModePanel;
	private final static String markLabel[] = {activeLabel[0],
		activeLabel[1], activeLabel[2], activeLabel[3]};
	private Canvas markCanvas[];
	private final static String clearLabel[] = {"Clear group", "Clear all"};
	private final static int G_CLEAR = 0;
	private final static int A_CLEAR = 1;
	private JButton clearButton[];
	/* mark resolution (node or subtree marking) */
	private final static JLabel resolutionTitle = new JLabel("Mark Resolution");
	private JPanel resolutionPanel;
	private ButtonGroup markResolution;
	private final static String resolutionLabel[] = {"Node", "Subtree"};
	private final static int N_RES = 0;
	private final static int S_RES = 1;
	private JRadioButton resolutionMode[];
	private ButtonGroup actionSelect;
	
	/* Least common ancestor checkboxes */
	private final static String LCA = "LCA";
	private JCheckBox LCAGroup; // for growing by LCA when checked
	public JCheckBox LCAMark; // for marking by LCA (follow marks to LCA) when checked
	
	private final static Color initialColor[] = {
		Color.getHSBColor(250f/360f, 1f, 1f),  // group A
		Color.getHSBColor(140f/360f, 1f, 1f),  // group B
		Color.getHSBColor(180f/360f, 1f, 1f),  // group C
		Color.getHSBColor(280f/360f, 1f, 1f),  // group D
		Color.getHSBColor(36f/360f, 1f, 1f),  // mouse over
		Color.getHSBColor(0f/360f, 1f, 1f),  // differences
		//new Color(87, 152, 64), // dark green
		Color.getHSBColor(310f/360f, 1f, 1f),  // found
		 new Color(87, 152, 64) // lca
	};
	private Color color[];
	


	/**
	 * Constructor for StateFrame.
	 * @throws HeadlessException
	 */
	public StateFrame(TreeJuxtaposer tj)  {
		super();
		this.tj = tj;
		this.setResizable(false);
		initComponents();
	}

	/**
	 * Constructor for StateFrame.
	 * @param gc
	 */
	public StateFrame(TreeJuxtaposer tj, GraphicsConfiguration gc) {
		super(gc);
		this.tj = tj;
		this.setResizable(false);
		initComponents();
	}

	/**
	 * Constructor for StateFrame.
	 * @param title
	 * @throws HeadlessException
	 */
	public StateFrame(TreeJuxtaposer tj, String title)  {
		super(title);
		this.tj = tj;		
		this.setResizable(false);
		initComponents();
	}

	/**
	 * Constructor for StateFrame.
	 * @param title
	 * @param gc
	 */
	public StateFrame(TreeJuxtaposer tj, String title, GraphicsConfiguration gc) {
		super(title, gc);
		this.tj = tj;		
		this.setResizable(false);
		initComponents();
	}

	private void initComponents()
	{
	GridBagConstraints gbc = new GridBagConstraints();
	this.setTitle(title);
	((JFrame)this).setLocation(10,30);
	this.getContentPane().setLayout(new GridBagLayout());

	/* Active Panel */
	activePanel = new JPanel(new GridBagLayout());
	activePanel.setBorder(javax.swing.BorderFactory.createLineBorder(Color.black));
	gbc.ipadx = 0;
	gbc.ipady = 0;
	gbc.gridx = 0;
	gbc.gridwidth = 1;
	gbc.gridheight = 1;
	gbc.gridy = 0;
	//activePanel.add(activePanelLabel, gbc);

	
	/* Grow direction panel */
	growDirectionPanel = new JPanel(new GridBagLayout());
	//growDirectionPanel.setBorder(javax.swing.BorderFactory.createLineBorder(Color.black));
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.gridwidth = 1;
	//gbc.gridwidth = growLabel.length;
	//growDirectionPanel.add(growDirLabel, gbc);
	growButton = new JButton[buttonLabel.length];
	for (int i = 0; i < buttonLabel.length; i++)
	{
		growButton[i] = new JButton(buttonLabel[i]);
		gbc.gridx = i;
		growDirectionPanel.add(growButton[i], gbc);
	}

	
	growMode = new JRadioButton[growLabel.length];
	growSelect = new ButtonGroup();
	gbc.gridy = 0;
	for (int i = 0; i < growLabel.length; i++)
	{
		growMode[i] = new JRadioButton(growLabel[i]);
		growSelect.add(growMode[i]);
		gbc.gridx = i + buttonLabel.length;
		growDirectionPanel.add(growMode[i], gbc);
	}
	
	LCAGroup = new JCheckBox(LCA, false);
	gbc.gridx = buttonLabel.length + 2*growLabel.length;
	gbc.anchor = GridBagConstraints.EAST;
//	growDirectionPanel.add(LCAGroup, gbc);
	gbc.anchor = GridBagConstraints.CENTER;
	
	resetButton = new JButton("Reset");
	gbc.gridx++;
	gbc.anchor = GridBagConstraints.WEST;
	growDirectionPanel.add(resetButton, gbc);
	gbc.anchor = GridBagConstraints.CENTER;
	
	gbc.gridx = 1;
	gbc.gridy = 1;
	gbc.fill = GridBagConstraints.BOTH;
	activePanel.add(growDirectionPanel, gbc);
	gbc.fill = GridBagConstraints.NONE;
	
	/* Grow button panel */
	/*
	growButtonPanel = new JPanel(new GridBagLayout());
	growButtonPanel.setBorder(javax.swing.BorderFactory.createLineBorder(Color.black));
	gbc.gridx = 0;
	gbc.gridy = 0;
	growButtonPanel.add(growButtonLabel);
	*/

	
	/* Active Group Panel */
	activeGroupPanel = new JPanel(new GridBagLayout());
	//activeGroupPanel.setBorder(javax.swing.BorderFactory.createLineBorder(Color.black));
	activeCanvas = new Canvas[activeLabel.length];
	activeMode = new JRadioButton[activeLabel.length];
	activeSelect = new ButtonGroup();
	color = new Color[activeLabel.length];
	gbc.gridy = 0;
	
	for (int i = 0; i < activeLabel.length; i++)
	{
		color[i] = new Color(initialColor[i].getRGB());
		if (i >= activeLabel.length / 2)
		{
			gbc.gridy = 1;
			gbc.gridx = 3 * (i - activeLabel.length/2 );
		}
		else
		  gbc.gridx = 3 * i;
		// put in radio buttons
		activeMode[i] = new JRadioButton(activeLabel[i]);
		activeSelect.add(activeMode[i]);
		gbc.ipadx = 0;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridx += 1;
		activeGroupPanel.add(activeMode[i], gbc);
		activeCanvas[i] = new Canvas();
		activeCanvas[i].setBackground(color[i]);
		activeCanvas[i].setSize(CANVAS_SIZE, CANVAS_SIZE);
		gbc.gridx -= 1;
		gbc.anchor = GridBagConstraints.EAST;
		JPanel canvasPanel = new JPanel();
		canvasPanel.add(activeCanvas[i]);
		canvasPanel.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
		activeGroupPanel.add(canvasPanel, gbc);
		//activeGroupPanel.add(activeCanvas[i], gbc);
		gbc.ipadx = 10;
		gbc.gridx += 2;
		blank = new JPanel();
		activeGroupPanel.add(blank, gbc);
	}
	gbc.anchor = GridBagConstraints.CENTER;
	gbc.ipadx = 0;
	activeMode[M_ACT].setSelected(true);
	gbc.gridy = 0;
	gbc.gridx = 1;
	gbc.fill = GridBagConstraints.BOTH;
	activePanel.add(activeGroupPanel, gbc);
	gbc.fill = GridBagConstraints.NONE;
	
	/* Active Panel events */
	resetButton.addActionListener(this);
	
	growMode[H_MODE].addActionListener(this);
	growMode[V_MODE].addActionListener(this);
	growMode[B_MODE].addActionListener(this);
	
	growButton[B_GROW].addActionListener(this);
	growButton[S_GROW].addActionListener(this);
	
	activeMode[GA_ACT].addActionListener(this);
	activeMode[GB_ACT].addActionListener(this);
	activeMode[GC_ACT].addActionListener(this);
	activeMode[GD_ACT].addActionListener(this);
	activeMode[M_ACT].addActionListener(this);
	activeMode[D_ACT].addActionListener(this);
	activeMode[F_ACT].addActionListener(this);
	activeMode[LCA_ACT].addActionListener(this);
	
	activeCanvas[GA_ACT].addMouseListener(this);
	activeCanvas[GB_ACT].addMouseListener(this);
	activeCanvas[GC_ACT].addMouseListener(this);
	activeCanvas[GD_ACT].addMouseListener(this);
	activeCanvas[M_ACT].addMouseListener(this);
	activeCanvas[D_ACT].addMouseListener(this);
	activeCanvas[F_ACT].addMouseListener(this);
	activeCanvas[LCA_ACT].addMouseListener(this);
	LCAGroup.addActionListener(this);

	/* Mark Panel */
	markPanel = new JPanel(new GridBagLayout());
	markPanel.setBorder(javax.swing.BorderFactory.createLineBorder(Color.black));
	gbc.ipadx = 0;
	gbc.ipady = 0;
	gbc.gridx = 0;
	gbc.gridwidth = 1;
	gbc.gridheight = 1;
	gbc.gridy = 0;
	gbc.anchor = GridBagConstraints.WEST;
	markPanel.add(markPanelLabel, gbc);
	gbc.anchor = GridBagConstraints.CENTER;
	
	markModePanel = new JPanel();
	//markModePanel.setBorder(javax.swing.BorderFactory.createLineBorder(Color.black));
	markSelect = new ButtonGroup();
	gbc.gridy = 1;
	markMode = new JRadioButton[markLabel.length];
	markCanvas = new Canvas[markLabel.length];
	for (int i = 0; i < markLabel.length; i++)
	{
		markMode[i] = new JRadioButton(markLabel[i]);
		markCanvas[i] = new Canvas();
		markCanvas[i].setBackground(color[i]);
		markCanvas[i].setSize(CANVAS_SIZE, CANVAS_SIZE);
		markSelect.add(markMode[i]);
		gbc.gridx = i * 3;
		gbc.ipadx = 0;
		gbc.anchor = GridBagConstraints.EAST;
		JPanel canvasPanel = new JPanel();
		canvasPanel.add(markCanvas[i]);
		canvasPanel.setBorder(javax.swing.BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
		markModePanel.add(canvasPanel, gbc);
		//markModePanel.add(markCanvas[i], gbc);
		gbc.gridx += 1;
		gbc.anchor = GridBagConstraints.WEST;
		markModePanel.add(markMode[i], gbc);
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.gridx += 1;
		gbc.ipadx = 10;
		markModePanel.add(new JPanel(), gbc);
	}
	gbc.gridx = 0;
	gbc.gridy = 1;
	gbc.gridwidth = 10;
	gbc.fill = GridBagConstraints.BOTH;
	markPanel.add(markModePanel, gbc);
	gbc.fill = GridBagConstraints.NONE;
	gbc.gridwidth = 1;
	gbc.ipadx = 0;
	clearButton = new JButton[clearLabel.length];
	gbc.gridy = 2;
	gbc.fill = GridBagConstraints.NONE;
	for (int i = 0; i < clearLabel.length; i++)
	{
		clearButton[i] = new JButton(clearLabel[i]);
		gbc.gridx = i;
		markPanel.add(clearButton[i], gbc);
	}
	gbc.gridx = clearLabel.length;
	markPanel.add(new JPanel(), gbc);

	resolutionPanel = new JPanel();
	//resolutionPanel.setBorder(javax.swing.BorderFactory.createLineBorder(Color.black));
	markResolution = new ButtonGroup();
	resolutionMode = new JRadioButton[resolutionLabel.length];
	gbc.gridy = 0;
	gbc.gridx = 0;
	resolutionPanel.add(resolutionTitle, gbc);
	for (int i = 0; i < resolutionLabel.length; i++)
	{
		resolutionMode[i] = new JRadioButton(resolutionLabel[i]);
		markResolution.add(resolutionMode[i]);
		gbc.gridx = i + 1;
		resolutionPanel.add(resolutionMode[i], gbc);
	}
	
	LCAMark = new JCheckBox(LCA, false);
	gbc.gridx = gbc.gridx + 1;
	gbc.anchor = GridBagConstraints.EAST;
	resolutionPanel.add(LCAMark, gbc);
	gbc.anchor = GridBagConstraints.CENTER;
		
	gbc.gridx = clearLabel.length + 1;
	gbc.gridy = 2;
	markPanel.add(resolutionPanel, gbc);

	/* Mark panel events */
	markCanvas[GA_ACT].addMouseListener(this);
	markCanvas[GB_ACT].addMouseListener(this);
	markCanvas[GC_ACT].addMouseListener(this);
	markCanvas[GD_ACT].addMouseListener(this);
	
	clearButton[G_CLEAR].addActionListener(this);
	clearButton[A_CLEAR].addActionListener(this);
	
	markMode[GA_ACT].addActionListener(this);
	markMode[GB_ACT].addActionListener(this);
	markMode[GC_ACT].addActionListener(this);
	markMode[GD_ACT].addActionListener(this);

	resolutionMode[N_RES].addActionListener(this);
	resolutionMode[S_RES].addActionListener(this);
	
	LCAMark.addActionListener(this);
	
	gbc.ipadx = 0;
	gbc.ipady = 0;
	gbc.gridx = 0;
	gbc.gridwidth = 1;
	gbc.gridheight = 1;
	gbc.gridy = 0;
	gbc.fill = GridBagConstraints.BOTH;
	getContentPane().add(activePanel, gbc);
	gbc.gridy = 1;
	getContentPane().add(markPanel, gbc);
	pack();
	
	}

	public void prepareToShow()
	{
		System.out.println("prepare to show (state frame)");
		if (tj == null || tj.treeDrawers == null)
			return;
		observe();
	}

	public void growModeAction(int mode)
	{
		growMode[mode].setSelected(true);
		Iterator tdIter = tj.treeDrawers.iterator();
		while (tdIter.hasNext())
		{
			AccordionTreeDrawerFinal atd = (AccordionTreeDrawerFinal)tdIter.next();
			if (mode == B_MODE)
				atd.setGrowDirection(AccordionTreeDrawerFinal.ALL);
			else if (mode == H_MODE)
				atd.setGrowDirection(AccordionTreeDrawerFinal.HORIZ);
			else if (mode == V_MODE)
				atd.setGrowDirection(AccordionTreeDrawerFinal.VERT);
		}
	}

	public void activeModeAction(int mode)
	{
		int tjmode = TreeJuxtaposer.flashGroup;
		switch (mode) {
			case GA_ACT: tjmode = TreeJuxtaposer.markGroup[GA_ACT]; break;
			case GB_ACT: tjmode = TreeJuxtaposer.markGroup[GB_ACT]; break;
			case GC_ACT: tjmode = TreeJuxtaposer.markGroup[GC_ACT]; break;
			case GD_ACT: tjmode = TreeJuxtaposer.markGroup[GD_ACT]; break;
			case M_ACT: tjmode = TreeJuxtaposer.flashGroup; break;
			case F_ACT: tjmode = TreeJuxtaposer.foundGroup; break;
			case D_ACT: tjmode = TreeJuxtaposer.diffGroup; break;
			case LCA_ACT: tjmode = TreeJuxtaposer.lcaGroup; break;
		};
		Iterator tdIter = tj.treeDrawers.iterator();
		while (tdIter.hasNext())
		{
			AccordionTreeDrawerFinal atd = (AccordionTreeDrawerFinal)tdIter.next();
			atd.setActionGroup(tjmode);
		}
	}

	public void activeMarkAction(int mode)
	{
		markMode[mode].setSelected(true);
		Iterator tdIter = tj.treeDrawers.iterator();
		while (tdIter.hasNext())
		{
			AccordionTreeDrawerFinal atd = (AccordionTreeDrawerFinal)tdIter.next();
			atd.setMarkGroup(TreeJuxtaposer.markGroup[mode]);
		}
		tj.setGroupPriority(TreeJuxtaposer.markGroup[mode]);
	}

	public void activeResolutionAction(int mode)
	{
		resolutionMode[mode].setSelected(true);
		int atdmode = AccordionTreeDrawerFinal.SUBTREE;
		if (mode == N_RES) atdmode = AccordionTreeDrawerFinal.NODE;
		Iterator tdIter = tj.treeDrawers.iterator();
		while (tdIter.hasNext())
		{
			AccordionTreeDrawerFinal atd = (AccordionTreeDrawerFinal)tdIter.next();
			atd.setSelectionResolution(atdmode);
		}
	}
	
	public void unmarkAllGroups()
	{
		tj.unmarkGroup(TreeJuxtaposer.markGroup[GA_ACT]);
		tj.unmarkGroup(TreeJuxtaposer.markGroup[GB_ACT]);
		tj.unmarkGroup(TreeJuxtaposer.markGroup[GC_ACT]);
		tj.unmarkGroup(TreeJuxtaposer.markGroup[GD_ACT]);
		tj.unmarkGroup(TreeJuxtaposer.lcaGroup);
		LCAMark.setSelected(false);
	}

	// reflect choices made by keyboard or by selecting this panel
	public void observe()
	{
		if (tj.treeDrawers == null || tj.treeDrawers.size() == 0)
			return;
		AccordionTreeDrawerFinal atd = (AccordionTreeDrawerFinal)tj.treeDrawers.get(0);
		
		if (atd.getActionGroup() == tj.markGroup[0])
			activeMode[GA_ACT].setSelected(true);
		else if (atd.getActionGroup() == tj.markGroup[1])
			activeMode[GB_ACT].setSelected(true);
		else if (atd.getActionGroup() == tj.markGroup[2])
			activeMode[GC_ACT].setSelected(true);
		else if (atd.getActionGroup() == tj.markGroup[3])
			activeMode[GD_ACT].setSelected(true);
		else if (atd.getActionGroup() == tj.foundGroup)
			activeMode[F_ACT].setSelected(true);
		else if (atd.getActionGroup() == tj.diffGroup)
			activeMode[D_ACT].setSelected(true);
		else if (atd.getActionGroup() == tj.flashGroup)
			activeMode[M_ACT].setSelected(true);
		else if (atd.getActionGroup() == tj.lcaGroup)
					activeMode[LCA_ACT].setSelected(true);	
		else
		{
			System.out.println("bug: active mode (growing target) not set");
			for (int i = GA_ACT; i <= LCA_ACT; i++)
				activeMode[i].setSelected(false);
		}

		if (atd.getGrowDirection() == AccordionTreeDrawerFinal.HORIZ)
			growMode[H_MODE].setSelected(true);
		else if (atd.getGrowDirection() == AccordionTreeDrawerFinal.VERT)
			growMode[V_MODE].setSelected(true);
		else if (atd.getGrowDirection() == AccordionTreeDrawerFinal.ALL)
			growMode[B_MODE].setSelected(true);
		else
		{
			System.out.println("bug: action target (grow dir) not set");
			for (int i = H_MODE; i <= B_MODE; i++)
			growMode[i].setSelected(false);
		}
		
		if (atd.getSelectionResolution() == AccordionTreeDrawerFinal.SUBTREE)
			resolutionMode[S_RES].setSelected(true);
		else if (atd.getSelectionResolution() == AccordionTreeDrawerFinal.NODE)
			resolutionMode[N_RES].setSelected(true);
		else
		{
			System.out.println("bug: action target (resolution mode) not set");
			for (int i = N_RES; i <= S_RES; i++)
			resolutionMode[i].setSelected(false);
		}
		
		if (atd.getMarkGroup() == tj.markGroup[0])
			markMode[GA_ACT].setSelected(true);
		else if (atd.getMarkGroup() == tj.markGroup[1])
			markMode[GB_ACT].setSelected(true);
		else if (atd.getMarkGroup() == tj.markGroup[2])
			markMode[GC_ACT].setSelected(true);
		else if (atd.getMarkGroup() == tj.markGroup[3])
			markMode[GD_ACT].setSelected(true);
		else
		{
			System.out.println("bug: mark mode not set");
			for (int i = GA_ACT; i <= GD_ACT; i++)
				markMode[i].setSelected(false);
		}
	}
	
	public void actionPerformed(ActionEvent evt)
	{
		Object obj = evt.getSource();
		if (obj == resetButton)
		{
			tj.reset();
		}
		else if (obj == growMode[H_MODE] ||
			obj == growMode[V_MODE] ||
			obj == growMode[B_MODE])
		{
			int modeNum = H_MODE;
			if (obj == growMode[V_MODE]) modeNum = V_MODE;
			else if (obj == growMode[B_MODE]) modeNum = B_MODE;
			growModeAction(modeNum);
		}
		else if (obj == growButton[B_GROW] ||
			obj == growButton[S_GROW])
		{
			if (tj.treeDrawers.size() == 0)
				return;
			AccordionTreeDrawerFinal atd = (AccordionTreeDrawerFinal)tj.treeDrawers.get(0);
			int numSteps = (atd.jump) ? 1 : atd.getNumAnimSteps();
			boolean horiz = (atd.getGrowDirection() != AccordionTreeDrawerFinal.VERT);
			boolean vert = (atd.getGrowDirection() != AccordionTreeDrawerFinal.HORIZ);
//			if (atd.linkednav) // it makes no sense to only update one drawer, which one is picked?
			{
				tj.resizeGroup(tj.getGroup(atd.getActionGroup()), numSteps, obj == growButton[B_GROW], horiz, vert);
				tj.requestRedrawAll();
			}
//			else 
//			{
//				tj.resizeGroup(tj.getGroup(atd.getActionGroup()), numSteps, growBigger, horiz, vert, atd);
//				atd.
//			}
		}
		else if (obj == activeMode[GA_ACT] ||
			obj == activeMode[GB_ACT] ||
			obj == activeMode[GC_ACT] ||
			obj == activeMode[GD_ACT] ||
			obj == activeMode[M_ACT] ||
			obj == activeMode[F_ACT] ||
			obj == activeMode[D_ACT] ||
			obj == activeMode[LCA_ACT])
		{
			int mode = GA_ACT;
			((JRadioButton)obj).setSelected(true);
			if (obj == activeMode[GB_ACT]) mode = GB_ACT;
			else if (obj == activeMode[GC_ACT]) mode = GC_ACT;
			else if (obj == activeMode[GD_ACT]) mode = GD_ACT;
			else if (obj == activeMode[M_ACT]) mode = M_ACT;
			else if (obj == activeMode[F_ACT]) mode = F_ACT;
			else if (obj == activeMode[D_ACT]) mode = D_ACT;
			else if (obj == activeMode[LCA_ACT]) mode = LCA_ACT;
			activeModeAction(mode);
			tj.requestRedrawAll();
		}	
		else if (obj == clearButton[G_CLEAR])
		{
			if (markMode[GA_ACT].isSelected()) tj.unmarkGroup(TreeJuxtaposer.markGroup[GA_ACT]);
			else if (markMode[GB_ACT].isSelected()) tj.unmarkGroup(TreeJuxtaposer.markGroup[GB_ACT]);
			else if (markMode[GC_ACT].isSelected()) tj.unmarkGroup(TreeJuxtaposer.markGroup[GC_ACT]);
			else if (markMode[GD_ACT].isSelected()) tj.unmarkGroup(TreeJuxtaposer.markGroup[GD_ACT]);
			else unmarkAllGroups();
			tj.requestRedrawAll();
			tj.changedMarks();
		}
		else if (obj == clearButton[A_CLEAR])
		{
			unmarkAllGroups();
			tj.lcaNode.clear();
			tj.requestRedrawAll();
			tj.changedMarks();
		}
		else if (obj == markMode[GA_ACT] ||
			obj == markMode[GB_ACT] ||
			obj == markMode[GC_ACT] ||
			obj == markMode[GD_ACT])
		{
			int mode = GA_ACT;
			if (obj == markMode[GB_ACT]) mode = GB_ACT;
			else if (obj == markMode[GC_ACT]) mode = GC_ACT;
			else if (obj == markMode[GD_ACT]) mode = GD_ACT;
			activeMarkAction(mode);
			tj.requestRedrawAll();
		}	
		else if (obj == resolutionMode[N_RES] ||
			obj == resolutionMode[S_RES])
		{
			if (obj == resolutionMode[N_RES])
				activeResolutionAction(N_RES);
			else
				activeResolutionAction(S_RES);
		}
		else if (obj == LCAMark)
		{
			if(LCAMark.isSelected())			
			    for(int i=0; i<tj.treeDrawers.size(); i++)
			    tj.doLCAGeom(LCA_ACT, (AccordionTreeDrawerFinal)tj.treeDrawers.get(i));			
			else
			     tj.unmarkGroup(LCA_ACT);

			tj.requestRedrawAll();
			
		}
		else if (obj == LCAGroup)
		{
			System.out.println("LCAGroup: " + LCAGroup.isSelected());
		}
	}

	public void setLCAMark(boolean on) { LCAMark.setSelected(on); }
	public boolean getLCAMark() { return LCAMark.isSelected(); }
	public void setLCAGroup(boolean on) { LCAGroup.setSelected(on); }
	public boolean getLCAGroup() { return LCAGroup.isSelected(); }

	public void mouseClicked(MouseEvent evt)
	{
		Object obj = evt.getSource();
		if (obj == activeCanvas[GA_ACT] || obj == markCanvas[GA_ACT] ||
			obj == activeCanvas[GB_ACT] || obj == markCanvas[GB_ACT] ||
			obj == activeCanvas[GC_ACT] || obj == markCanvas[GC_ACT] ||
			obj == activeCanvas[GD_ACT] || obj == markCanvas[GD_ACT] ||
			obj == activeCanvas[M_ACT] ||
			obj == activeCanvas[D_ACT] ||
			obj == activeCanvas[F_ACT] ||
		    obj == activeCanvas[LCA_ACT]) 
		{
			int group = GA_ACT;
			int tjgroup = TreeJuxtaposer.markGroup[GA_ACT];
			if (obj == activeCanvas[GB_ACT] || obj == markCanvas[GB_ACT])
				{ group = GB_ACT; tjgroup = TreeJuxtaposer.markGroup[GB_ACT]; }
			else if (obj == activeCanvas[GC_ACT] || obj == markCanvas[GC_ACT])
				{ group = GC_ACT; tjgroup = TreeJuxtaposer.markGroup[GC_ACT]; }
			else if (obj == activeCanvas[GD_ACT] || obj == markCanvas[GD_ACT])
				{ group = GD_ACT; tjgroup = TreeJuxtaposer.markGroup[GD_ACT]; }
			else if (obj == activeCanvas[M_ACT])
				{ group = M_ACT; tjgroup = TreeJuxtaposer.flashGroup; }
			else if (obj == activeCanvas[D_ACT])
				{ group = D_ACT; tjgroup = TreeJuxtaposer.diffGroup; }
			else if (obj == activeCanvas[F_ACT])
				{ group = F_ACT; tjgroup = TreeJuxtaposer.foundGroup; } 
			else if (obj == activeCanvas[LCA_ACT])
				{ group = LCA_ACT; tjgroup = TreeJuxtaposer.lcaGroup; } 	
			Color newColor = JColorChooser.showDialog(activePanel, colorTitle[group], color[group]);
			if (newColor != null)
			{
				color[group] = newColor;
				activeCanvas[group].setBackground(color[group]);
				if (group == GA_ACT || group == GB_ACT || group == GC_ACT || group == GD_ACT)
					markCanvas[group].setBackground(color[group]);
				tj.setGroupColor(tjgroup, color[group]);
				tj.requestRedrawAll();
				tj.changedMarks();
			}
		}
	}

	public void mouseEntered(MouseEvent evt) {}
	public void mouseExited(MouseEvent evt) {}
	public void mousePressed(MouseEvent evt) {}
	public void mouseReleased(MouseEvent evt) {}
	
}
